<cfcomponent>
    <cffunction name="getradiuszips" description="Returns a query of zipcodes in a surrounding area" returntype="query">
    	<cfargument name="zipcode" default="43512" required="yes" hint="zip code to get surrounding zip codes for">
        <cfargument name="radius" default="50" required="No" hint="Radius in miles to search within.  Default is 100.">
        <cfargument name="dsn" required="Yes" hint="Specific DSN to use where the zips table is stored.">
		<cfargument name="databasetype" required="No" default="mysql" hint="Default is mysql, but you can specifcy mssql to run the mssql version of the query.">
       
       
         <cfquery name="qzip" datasource="#dsn#" maxrows="1">
         SELECT zipcode, lat, lon, city, state 
         FROM zips 
         WHERE zipcode = <cfqueryparam value="#ZipCode#" cfsqltype="cf_sql_varchar">
         </cfquery>

		<cfif databasetype IS 'mysql'>
        <cfquery name="qsearch" datasource="#dsn#">
            SELECT zipcode, lat, lon, city, state,
            ROUND((ACOS((SIN(#qzip.lat#/57.2958) * SIN(lat/57.2958)) + (COS(#qzip.lat#/57.2958) * COS(lat/57.2958) * COS(lon/57.2958 - #qzip.lon#/57.2958)))) * 3963) AS distance
            FROM zips
            WHERE (lat >= #qzip.lat# - (#radius#/111)) 
            AND (lat <= #qzip.lat# + (#radius#/111))
            AND (lon >= #qzip.lon# - (#radius#/111))
            AND (lon <= #qzip.lon# + (#radius#/111))
            ORDER BY distance
        </cfquery>
		</cfif>
		<cfif databasetype IS 'mssql'>
        <cfquery name="qsearch" datasource="#dsn#">
            SELECT zipcode, lat, lon, city, state,
            ROUND((ACOS((SIN(#qzip.lat#/57.2958) * SIN(lat/57.2958)) + (COS(#qzip.lat#/57.2958) * COS(lat/57.2958) * COS(lon/57.2958 - #qzip.lon#/57.2958)))) * 3963,3) AS distance
            FROM zips
            WHERE (lat >= #qzip.lat# - (#radius#/111))
            AND (lat <= #qzip.lat# + (#radius#/111))
            AND (lon >= #qzip.lon# - (#radius#/111))
            AND (lon <= #qzip.lon# + (#radius#/111))
            ORDER BY distance
        </cfquery>
		</cfif>        
		<cfreturn qsearch>        
    </cffunction>
    
    <cffunction name="getdistance" description="gets the distance in miles between two zip codes" returntype="numeric">
    	<cfargument name="fromzip" default="43512" required="yes" hint="zip code to measure from">
        <cfargument name="tozip" default="43506" required="yes" hint="zip code to measure to.">
        <cfargument name="dsn" required="Yes" hint="Specific DSN to use.">

        <cfquery name="qfromzip" datasource="#dsn#">
        SELECT * From zips WHERE zipcode = '#fromzip#'
        </cfquery>
        
        <cfquery name="qtozip" datasource="#dsn#">
        SELECT * From zips WHERE zipcode = '#tozip#'
        </cfquery>
		        
        <cfif qtozip.recordcount GT 0 AND qfromzip.recordcount GT 0>
            <cfset x = 69.1 * (qtozip.lat - qfromzip.lat)>
            <cfset y = 69.1 * (qtozip.lon - qfromzip.lon) * cos(qfromzip.lat / 57.3)>
            <cfset distance = SQR(x * x + y * y)>
		<cfelse>
        	<!---invalid so just set it out of range--->
        	<cfset distance = 99999>
		</cfif>

		<cfreturn distance>        
    </cffunction>
</cfcomponent>